@@ -26,7 +26,7 @@ static void exit_single_user(MVMThreadContext *tc, MVMArrayBody *arr) {
 static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
     MVMSTable        *st = MVM_gc_allocate_stable(tc, &VMArray_this_repr, HOW);
 
-    MVMROOT(tc, st, {
+    MVMROOT(tc, st) {
         MVMObject *obj = MVM_gc_allocate_type_object(tc, st);
         MVMArrayREPRData *repr_data = (MVMArrayREPRData *)MVM_malloc(sizeof(MVMArrayREPRData));
 
@@ -37,7 +37,7 @@ static MVMObject * type_object_for(MVMThreadContext *tc, MVMObject *HOW) {
         MVM_ASSIGN_REF(tc, &(st->header), st->WHAT, obj);
         st->size = sizeof(MVMArray);
         st->REPR_data = repr_data;
-    });
+    }
 
     return st->WHAT;
 }
@@ -143,6 +143,100 @@ static const MVMStorageSpec * get_storage_spec(MVMThreadContext *tc, MVMSTable *
     return &storage_spec;
 }
 
+void MVM_VMArray_at_pos_s(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister *value) {
+    MVMArrayREPRData *repr_data = (MVMArrayREPRData *)st->REPR_data;
+    MVMArrayBody     *body      = (MVMArrayBody *)data;
+
+    /* Handle negative indexes. */
+    if (index < 0) {
+        index += body->elems;
+        if (index < 0)
+            MVM_exception_throw_adhoc(tc, "MVMArray: Index out of bounds");
+    }
+
+    /* Go by type. */
+    if (repr_data->slot_type != MVM_ARRAY_STR)
+        MVM_exception_throw_adhoc(tc, "MVMArray: atpos expected a string register, but %u is not MVM_ARRAY_STR", repr_data->slot_type);
+    if (index >= body->elems)
+        value->s = NULL;
+    else
+        value->s = body->slots.s[body->start + index];
+}
+
+void MVM_VMArray_at_pos_i(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister *value) {
+    MVMArrayBody     *body      = (MVMArrayBody *)data;
+
+    /* Handle negative indexes. */
+    if (index < 0) {
+        index += body->elems;
+        if (index < 0)
+            MVM_exception_throw_adhoc(tc, "MVMArray: Index out of bounds");
+    }
+
+    /* Go by type. */
+    if (index >= body->elems)
+        value->i64 = 0;
+    else
+        value->i64 = (MVMint64)body->slots.i64[body->start + index];
+}
+
+void MVM_VMArray_at_pos_u(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister *value) {
+    MVMArrayBody     *body      = (MVMArrayBody *)data;
+
+    /* Handle negative indexes. */
+    if (index < 0) {
+        index += body->elems;
+        if (index < 0)
+            MVM_exception_throw_adhoc(tc, "MVMArray: Index out of bounds");
+    }
+
+    /* Go by type. */
+    if (index >= body->elems)
+        value->u64 = 0;
+    else
+        value->u64 = (MVMuint64)body->slots.u64[body->start + index];
+}
+
+void MVM_VMArray_at_pos_n(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister *value) {
+    MVMArrayBody     *body      = (MVMArrayBody *)data;
+
+    /* Handle negative indexes. */
+    if (index < 0) {
+        index += body->elems;
+        if (index < 0)
+            MVM_exception_throw_adhoc(tc, "MVMArray: Index out of bounds");
+    }
+
+    /* Go by type. */
+    if (index >= body->elems)
+        value->n64 = 0;
+    else
+        value->n64 = (MVMnum64)body->slots.n64[body->start + index];
+}
+
+void MVM_VMArray_at_pos_o(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister *value) {
+    MVMArrayREPRData *repr_data = (MVMArrayREPRData *)st->REPR_data;
+    MVMArrayBody     *body      = (MVMArrayBody *)data;
+
+    /* Handle negative indexes. */
+    if (index < 0) {
+        index += body->elems;
+        if (index < 0)
+            MVM_exception_throw_adhoc(tc, "MVMArray: Index out of bounds");
+    }
+
+    /* Go by type. */
+    if (repr_data->slot_type != MVM_ARRAY_OBJ)
+        MVM_exception_throw_adhoc(tc, "MVMArray: atpos with an object register, but array type %u is not MVM_ARRAY_OBJ", repr_data->slot_type);
+    if (index >= body->elems) {
+        value->o = tc->instance->VMNull;
+    }
+    else {
+        MVMObject *found = body->slots.o[body->start + index];
+        value->o = found ? found : tc->instance->VMNull;
+    }
+}
+
 void MVM_VMArray_at_pos(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 index, MVMRegister *value, MVMuint16 kind) {
     MVMArrayREPRData *repr_data = (MVMArrayREPRData *)st->REPR_data;
     MVMArrayBody     *body      = (MVMArrayBody *)data;
@@ -909,10 +1003,12 @@ static void write_buf(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void
     /* resize the array if necessary*/
     size_t elem_size = repr_data->elem_size;
     /* No need to account for start, set_size_internal will take care of that */
-    if (elems * elem_size < offset * elem_size + count)
+    if (elems * elem_size < offset * elem_size + count) {
         set_size_internal(tc, body, offset + count, repr_data);
+        start = body->start;
+    }
 
-    memcpy(body->slots.u8 + (start + offset) * repr_data->elem_size, from, count);
+    memcpy(body->slots.u8 + (start + offset) * elem_size, from, count);
 }
 
 static MVMint64 read_buf(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, void *data, MVMint64 offset, MVMuint64 count) {
@@ -935,7 +1031,7 @@ static MVMint64 read_buf(MVMThreadContext *tc, MVMSTable *st, MVMObject *root, v
 #if MVM_BIGENDIAN
 	+ (8 - count)
 #endif
-	, body->slots.u8 + (start + offset) * repr_data->elem_size, count);
+	, body->slots.u8 + (start + offset) * elem_size, count);
     return result;
 }
 
